@@ -206,6 +206,68 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:          :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def address_set_add(self, name, addresses=None, may_exist=False):
+        """Create a set of addresses named 'name'
+
+        :param name:      The name of the address set
+        :type name:       string
+        :param addresses: One or more IP addresses to add to the address set
+        :type addresses:  list of strings
+        :param may_exist: If True, don't fail if the address set already exists
+        :type may_exist:  boolean
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_del(self, address_set, if_exists=False):
+        """Delete a set of addresses 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param if_exists:   Do not fail if the Address_set row does not exist
+        :type if_exists:    bool
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_get(self, address_set):
+        """Get a set of addresses for 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_list(self):
+        """Get all sets of addresses
+
+        :returns: :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def address_set_add_addresses(self, address_set, addresses):
+        """Add a list of addresses to 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to add to the address set
+        :type addresses:    string or list of strings
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_remove_addresses(self, address_set, addresses):
+        """Remove a list of addresses from 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to remove from address set
+        :type addresses:    string of list of strings
+        :returns:           :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def qos_add(self, switch, direction, priority, match, rate=None,
                 burst=None, dscp=None, may_exist=False, **columns):
@@ -547,6 +609,15 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lrp_get(self, port):
+        """Get logical router port for 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def lrp_set_enabled(self, port, is_enabled):
         """Set administrative state of 'port'
@@ -587,6 +658,70 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:    :class:`Command` with dict result
         """
 
+    @abc.abstractmethod
+    def lrp_set_gateway_chassis(self, port, gateway_chassis, priority=0):
+        """Set gateway chassis for 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param priority:        The priority of the gateway chassis
+        :type priority:         int
+        :returns:               :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_get_gateway_chassis(self, port):
+        """Get the names of all gateway chassis on 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :type port:  int
+        :returns:    :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def lrp_del_gateway_chassis(self, port, gateway_chassis, if_exists=False):
+        """Delete gateway chassis from 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param if_exists:       If True, don't fail if the gateway chassis
+                                doesn't exist
+        :type if_exists:        boolean
+        :returns:               :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_add_networks(self, port, networks, may_exist=False):
+        """Add a network to 'port'
+
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param networks:  One or more IP address/netmask to assign to the port
+        :type networks:   list or string
+        :param may_exist: If True, don't fail if the networks already exists
+        :type may_exist:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_del_networks(self, port, networks, if_exists=False):
+        """Remove a network from 'port'
+
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param networks:  One or more IP address/netmask to remove from
+                          the port
+        :type networks:   list or string
+        :param if_exists: If True, don't fail if the networks doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
@@ -689,6 +824,53 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lr_policy_add(self, router, priority, match, action, may_exist=False,
+                      **columns):
+        """Add a routing policy (PBR) to 'router'
+
+        :param router:    The name or uuid of the router
+        :type router:     string or uuid.UUID
+        :param priority:  The priority of the policy
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :param action:    The action to take upon match
+        :type action:     POLICY_ACTION_ALLOW, POLICY_ACTION_DROP,
+                          POLICY_ACTION_REROUTE
+        :param may_exist: If True, don't fail if the policy already exists
+                          and instead update the columns
+        :type may_exist:  boolean
+        :param columns:   Additional columns to directly set on the policy
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lr_policy_del(self, router, priority=None, match=None,
+                      if_exists=False):
+        """Remove routing policies (PBR) from router
+
+        :param router:     The name or uuid of the router
+        :type router:      string or uuid.UUID
+        :param priority:   The priority of the rule
+        :type priority:    int
+        :param match:      The match condition of the rule. This should only
+                           be specified if priority is specified.
+        :type match:       string
+        :param if_exists:  If True, don't fail if the port doesn't exist
+        :type if_exists:   boolean
+        :returns:          :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lr_policy_list(self, router):
+        """Get the list of routing policies on 'router'
+
+        :param router: The name or uuid of the router
+        :type router:  string or uuid.UUID
+        :returns:      :class:`Command` with RowView list result
+        """
+
     @abc.abstractmethod
     def lb_add(self, lb, vip, ips, protocol=const.PROTO_TCP, may_exist=False,
                **columns):
@@ -730,6 +912,88 @@ class API(api.API, metaclass=abc.ABCMeta):
     def lb_list(self):
         """Get the UUIDs of all load balanacers"""
 
+    @abc.abstractmethod
+    def lb_get(self, lb):
+        """Get load balancer for 'lb'
+
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lb_add_health_check(self, lb, vip, **options):
+        """Add health check for 'lb'
+
+        :param lb:      The name or uuid of a load balancer
+        :type lb:       string or uuid.UUID
+        :param vip:     A virtual IP
+        :type vip:      string
+        :param options: keys and values for the port 'options' dict
+        :type options:  key: string, value: string
+        :returns:       :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_del_health_check(self, lb, hc_uuid, if_exists=False):
+        """Remove health check from 'lb'
+
+        :param lb:        The name or uuid of a load balancer
+        :type lb:         string or uuid.UUID
+        :param hc_uuid:   uuid of a health check
+        :type hc_uuid:    uuid.UUID
+        :param if_exists: If True, don't fail if the hc_uuid doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_add_ip_port_mapping(self, lb, endpoint_ip, port_name, source_ip):
+        """Add IP port mapping to 'lb'
+
+        Maps from endpoint IP to a colon-separated pair of logical port name
+        and source IP, e.g. port_name:sourc_ip.
+
+        :param lb:          The name or uuid of a load balancer
+        :type lb:           string or uuid.UUID
+        :param endpoint_ip: IPv4 address
+        :type endpoint_ip:  string
+        :param port_name:   The name of a logical port
+        :type port_name:    string
+        :param source_ip:   IPv4 address
+        :type source_ip:    string
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lb_del_ip_port_mapping(self, lb, endpoint_ip):
+        """Remove IP port mapping from 'lb'
+
+        :param lb:          The name or uuid of a load balancer
+        :type lb:           string or uuid.UUID
+        :param endpoint_ip: IPv4 address
+        :type endpoint_ip:  string
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def health_check_set_options(self, hc_uuid, **options):
+        """Set options to the 'health_check'
+
+        :param hc_uuid: uuid of the health check
+        :type hc_uuid:  uuid.UUID
+        :param options: keys and values for the port 'options' dict
+        :type options:  key: string, value: string
+        :returns:       :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def health_check_get_options(self, hc_uuid):
+        """Get the options for 'health_check'
+
+        :param hc_uuid: uuid of the health check
+        :type hc_uuid:  uuid.UUID
+        :returns:       :class:`Command` with dict result
+        """
+
     @abc.abstractmethod
     def lr_lb_add(self, router, lb, may_exist=False):
         """Add a load-balancer to 'router'
