@@ -91,8 +91,10 @@ class _AclAddHelper(cmd.AddCommand):
         if not 0 <= priority <= const.ACL_PRIORITY_MAX:
             raise ValueError("priority must be between 0 and %s, inclusive" % (
                              const.ACL_PRIORITY_MAX))
-        if action not in ('allow', 'allow-related', 'drop', 'reject'):
-            raise TypeError("action must be allow/allow-related/drop/reject")
+        if action not in ('allow', 'allow-related', 'allow-stateless',
+                          'drop', 'reject'):
+            raise TypeError("action must be allow/allow-related/"
+                            "allow-stateless/drop/reject")
         super().__init__(api)
         self.entity = entity
         self.direction = direction
@@ -212,6 +214,88 @@ class PgAclListCommand(_AclListHelper):
     lookup_table = 'Port_Group'
 
 
+class AddressSetAddCommand(cmd.AddCommand):
+    table_name = 'Address_Set'
+
+    def __init__(self, api, name, addresses=None, may_exist=False):
+        super().__init__(api)
+        self.name = name
+        self.addresses = [str(netaddr.IPAddress(address))
+                          for address in addresses or []]
+        self.may_exist = may_exist
+
+    def run_idl(self, txn):
+        address_set = self.api.lookup(self.table_name, self.name, None)
+        if address_set:
+            if self.may_exist:
+                self.result = rowview.RowView(address_set)
+                return
+            raise RuntimeError("Address set %s exists" % self.name)
+
+        address_set = txn.insert(self.api.tables[self.table_name])
+        address_set.name = self.name
+        if self.addresses:
+            address_set.addresses = self.addresses
+        self.result = address_set.uuid
+
+
+class AddressSetDelCommand(cmd.BaseCommand):
+    table_name = 'Address_Set'
+
+    def __init__(self, api, address_set, if_exists=False):
+        super().__init__(api)
+        self.address_set = address_set
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        try:
+            address_set = self.api.lookup(self.table_name, self.address_set)
+            address_set.delete()
+        except idlutils.RowNotFound as e:
+            if self.if_exists:
+                return
+            msg = "Address set %s does not exist" % self.address_set
+            raise RuntimeError(msg) from e
+
+
+class AddressSetGetCommand(cmd.BaseGetRowCommand):
+    table = 'Address_Set'
+
+
+class AddressSetListCommand(cmd.ReadOnlyCommand):
+    table = 'Address_Set'
+
+    def run_idl(self, txn):
+        self.result = [rowview.RowView(r) for
+                       r in self.api.tables[self.table].rows.values()]
+
+
+class AddressSetUpdateAddressesCommand(cmd.BaseCommand):
+    table_name = 'Address_Set'
+
+    def __init__(self, api, address_set, addresses):
+        super().__init__(api)
+        self.address_set = address_set
+        if isinstance(addresses, (str, bytes)):
+            addresses = [addresses]
+        self.addresses = [str(netaddr.IPAddress(address))
+                          for address in addresses]
+
+
+class AddressSetAddAddressesCommand(AddressSetUpdateAddressesCommand):
+    def run_idl(self, txn):
+        address_set = self.api.lookup(self.table_name, self.address_set)
+        for address in self.addresses:
+            address_set.addvalue('addresses', address)
+
+
+class AddressSetRemoveAddressCommand(AddressSetUpdateAddressesCommand):
+    def run_idl(self, txn):
+        address_set = self.api.lookup(self.table_name, self.address_set)
+        for address in self.addresses:
+            address_set.delvalue('addresses', address)
+
+
 class QoSAddCommand(cmd.AddCommand):
     table_name = 'QoS'
 
@@ -361,8 +445,7 @@ class QoSDelExtIdCommand(cmd.BaseCommand):
 
     def run_idl(self, txn):
         try:
-            lswitch = idlutils.row_by_value(self.api.idl, 'Logical_Switch',
-                                            'name', self.lswitch)
+            lswitch = self.api.lookup('Logical_Switch', self.lswitch)
         except idlutils.RowNotFound as e:
             if self.if_exists:
                 return
@@ -612,30 +695,13 @@ class LspGetTypeCommand(cmd.ReadOnlyCommand):
         self.result = lsp.type
 
 
-class LspSetOptionsCommand(cmd.BaseCommand):
+class LspSetOptionsCommand(cmd.BaseSetOptionsCommand):
     table = 'Logical_Switch_Port'
 
-    def __init__(self, api, port, **options):
-        super().__init__(api)
-        self.port = port
-        self.options = options
 
-    def run_idl(self, txn):
-        lsp = self.api.lookup(self.table, self.port)
-        lsp.options = self.options
-
-
-class LspGetOptionsCommand(cmd.ReadOnlyCommand):
+class LspGetOptionsCommand(cmd.BaseGetOptionsCommand):
     table = 'Logical_Switch_Port'
 
-    def __init__(self, api, port):
-        super().__init__(api)
-        self.port = port
-
-    def run_idl(self, txn):
-        lsp = self.api.lookup(self.table, self.port)
-        self.result = lsp.options
-
 
 class LspSetDhcpV4OptionsCommand(cmd.BaseCommand):
     def __init__(self, api, port, dhcpopt_uuid):
@@ -695,25 +761,12 @@ class DhcpOptionsGetCommand(cmd.BaseGetRowCommand):
     table = 'DHCP_Options'
 
 
-class DhcpOptionsSetOptionsCommand(cmd.BaseCommand):
-    def __init__(self, api, dhcpopt_uuid, **options):
-        super().__init__(api)
-        self.dhcpopt_uuid = dhcpopt_uuid
-        self.options = options
-
-    def run_idl(self, txn):
-        dhcpopt = self.api.lookup('DHCP_Options', self.dhcpopt_uuid)
-        dhcpopt.options = self.options
+class DhcpOptionsSetOptionsCommand(cmd.BaseSetOptionsCommand):
+    table = 'DHCP_Options'
 
 
-class DhcpOptionsGetOptionsCommand(cmd.ReadOnlyCommand):
-    def __init__(self, api, dhcpopt_uuid):
-        super().__init__(api)
-        self.dhcpopt_uuid = dhcpopt_uuid
-
-    def run_idl(self, txn):
-        dhcpopt = self.api.lookup('DHCP_Options', self.dhcpopt_uuid)
-        self.result = dhcpopt.options
+class DhcpOptionsGetOptionsCommand(cmd.BaseGetOptionsCommand):
+    table = 'DHCP_Options'
 
 
 class LrAddCommand(cmd.BaseCommand):
@@ -848,6 +901,10 @@ class LrpListCommand(cmd.ReadOnlyCommand):
         self.result = [rowview.RowView(r) for r in router.ports]
 
 
+class LrpGetCommand(cmd.BaseGetRowCommand):
+    table = 'Logical_Router_Port'
+
+
 class LrpSetEnabledCommand(cmd.BaseCommand):
     def __init__(self, api, port, is_enabled):
         super().__init__(api)
@@ -870,19 +927,108 @@ class LrpGetEnabledCommand(cmd.ReadOnlyCommand):
         self.result = next(iter(lrp.enabled), True)
 
 
-class LrpSetOptionsCommand(LspSetOptionsCommand):
+class LrpSetOptionsCommand(cmd.BaseSetOptionsCommand):
     table = 'Logical_Router_Port'
 
 
-class LrpGetOptionsCommand(LspGetOptionsCommand):
+class LrpGetOptionsCommand(cmd.BaseGetOptionsCommand):
     table = 'Logical_Router_Port'
 
 
+class LrpSetGatewayChassisCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, gateway_chassis, priority):
+        super().__init__(api)
+        self.port = port
+        self.gateway_chassis = gateway_chassis
+        self.priority = priority
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        gwc_name = '%s_%s' % (lrp.name, self.gateway_chassis)
+        cmd = GatewayChassisAddCommand(self.api, gwc_name,
+                                       self.gateway_chassis,
+                                       self.priority, may_exist=True)
+        cmd.run_idl(txn)
+        lrp.addvalue('gateway_chassis', cmd.result)
+
+
+class LrpGetGatewayChassisCommand(cmd.ReadOnlyCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port):
+        super().__init__(api)
+        self.port = port
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        self.result = [rowview.RowView(d) for d in lrp.gateway_chassis]
+
+
+class LrpDelGatewayChassisCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, gateway_chassis, if_exists=False):
+        super().__init__(api)
+        self.port = port
+        self.gateway_chassis = gateway_chassis
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for gwc in lrp.gateway_chassis:
+            if gwc.chassis_name == self.gateway_chassis:
+                lrp.delvalue('gateway_chassis', gwc)
+                return
+
+        if not self.if_exists:
+            msg = "Gateway chassis '%s' on port %s does not exist" % (
+                self.gateway_chassis, self.port)
+            raise RuntimeError(msg)
+
+
+class _LrpNetworksCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, networks, exists):
+        super().__init__(api)
+        self.port = port
+        self.exists = exists
+        if isinstance(networks, (str, bytes)):
+            networks = [networks]
+        self.networks = [str(netaddr.IPNetwork(network))
+                         for network in networks]
+
+
+class LrpAddNetworksCommand(_LrpNetworksCommand):
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for network in self.networks:
+            if network in lrp.networks and not self.exists:
+                msg = "Network '%s' already exist in networks of port %s" % (
+                    network, lrp.uuid)
+                raise RuntimeError(msg)
+            lrp.addvalue('networks', network)
+
+
+class LrpDelNetworksCommand(_LrpNetworksCommand):
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for network in self.networks:
+            if network not in lrp.networks and not self.exists:
+                msg = "Network '%s' does not exist in networks of port %s" % (
+                    network, lrp.uuid)
+                raise RuntimeError(msg)
+            lrp.delvalue('networks', network)
+
+
 class LrRouteAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, prefix, nexthop, port=None,
                  policy='dst-ip', may_exist=False):
         prefix = str(netaddr.IPNetwork(prefix))
-        nexthop = str(netaddr.IPAddress(nexthop))
+        if nexthop != const.ROUTE_DISCARD:
+            nexthop = str(netaddr.IPAddress(nexthop))
         super().__init__(api)
         self.router = router
         self.prefix = prefix
@@ -902,7 +1048,7 @@ class LrRouteAddCommand(cmd.BaseCommand):
                 route.nexthop = self.nexthop
                 route.policy = self.policy
                 if self.port:
-                    route.port = self.port
+                    route.output_port = self.port
                 self.result = rowview.RowView(route)
                 return
         route = txn.insert(self.api.tables['Logical_Router_Static_Route'])
@@ -910,7 +1056,7 @@ class LrRouteAddCommand(cmd.BaseCommand):
         route.nexthop = self.nexthop
         route.policy = self.policy
         if self.port:
-            route.port = self.port
+            route.output_port = self.port
         lr.addvalue('static_routes', route)
         self.result = route.uuid
 
@@ -1074,6 +1220,96 @@ class LrNatListCommand(cmd.ReadOnlyCommand):
         self.result = [rowview.RowView(r) for r in lr.nat]
 
 
+class LrPolicyAddCommand(cmd.AddCommand):
+    table_name = 'Logical_Router_Policy'
+
+    def __init__(self, api, router, priority, match, action, may_exist=False,
+                 **columns):
+        if not 0 <= priority <= const.LR_POLICY_PRIORITY_MAX:
+            raise ValueError("priority must be between 0 and %s, inclusive"
+                             % (const.LR_POLICY_PRIORITY_MAX,))
+        if action not in const.POLICY_ACTION_TYPES:
+            raise TypeError("action not in %s" % (const.POLICY_ACTION_TYPES,))
+        if (action == const.POLICY_ACTION_REROUTE and not
+                (columns.get("nexthop") or columns.get("nexthops"))):
+            raise ValueError("must specify nexthop(s) for action %s"
+                             % (const.POLICY_ACTION_REROUTE,))
+        if ((columns.get("nexthop") or columns.get("nexthops")) and
+                action != const.POLICY_ACTION_REROUTE):
+            raise ValueError("nexthop only valid for action %s"
+                             % (const.POLICY_ACTION_REROUTE,))
+        super().__init__(api)
+        self.router = router
+        self.priority = priority
+        self.match = match
+        self.action = action
+        self.may_exist = may_exist
+        self.columns = columns
+
+    def run_idl(self, txn):
+        lr = self.api.lookup('Logical_Router', self.router)
+        for policy in lr.policies:
+            if (self.priority, self.match) == (policy.priority, policy.match):
+                if self.may_exist:
+                    policy.action = self.action
+                    self.set_columns(policy, **self.columns)
+                    self.result = rowview.RowView(policy)
+                    return
+                raise RuntimeError("Logical_Router Policy already exists")
+        policy = txn.insert(self.api.tables[self.table_name])
+        policy.priority = self.priority
+        policy.match = self.match
+        policy.action = self.action
+        self.set_columns(policy, **self.columns)
+        lr.addvalue('policies', policy)
+        self.result = policy.uuid
+
+
+class LrPolicyDelCommand(cmd.BaseCommand):
+    def __init__(self, api, router, priority=None, match=None,
+                 if_exists=False):
+        self.conditions = []
+        if priority is not None:
+            if not 0 <= priority <= const.LR_POLICY_PRIORITY_MAX:
+                raise ValueError("priority must be between 0 and %s, inclusive"
+                                 % (const.LR_POLICY_PRIORITY_MAX,))
+            self.conditions += [('priority', '=', priority)]
+            if match is not None:
+                self.conditions += [('match', '=', match)]
+        elif match:
+            raise TypeError("must specify priority with match")
+        super().__init__(api)
+        self.router = router
+        self.priority = priority
+        self.match = match
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lr = self.api.lookup('Logical_Router', self.router)
+        found = False
+        for policy in lr.policies:
+            if idlutils.row_match(policy, self.conditions):
+                found = True
+                lr.delvalue('policies', policy)
+                policy.delete()
+                if self.match:
+                    return
+        if self.match and not (found or self.if_exists):
+            raise RuntimeError(
+                "Policy with match %s and priority %s does not exist in "
+                "router %s" % (self.match, self.priority, self.router))
+
+
+class LrPolicyListCommand(cmd.ReadOnlyCommand):
+    def __init__(self, api, router):
+        super().__init__(api)
+        self.router = router
+
+    def run_idl(self, txn):
+        lr = self.api.lookup('Logical_Router', self.router)
+        self.result = [rowview.RowView(r) for r in lr.policies]
+
+
 class LbAddCommand(cmd.BaseCommand):
     def __init__(self, api, lb, vip, ips, protocol=const.PROTO_TCP,
                  may_exist=False, **columns):
@@ -1140,6 +1376,100 @@ class LbListCommand(cmd.ReadOnlyCommand):
                        for r in self.api.tables['Load_Balancer'].rows.values()]
 
 
+class LbGetCommand(cmd.BaseGetRowCommand):
+    table = 'Load_Balancer'
+
+
+class LbAddHealthCheckCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, vip, **options):
+        super().__init__(api)
+        self.lb = lb
+        self.vip = vip
+        self.options = options
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        cmd = HealthCheckAddCommand(self.api, self.vip, **self.options)
+        cmd.run_idl(txn)
+        lb.addvalue('health_check', cmd.result)
+
+
+class LbDelHealthCheckCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, hc_uuid, if_exists=False):
+        super().__init__(api)
+        self.lb = lb
+        self.hc_uuid = hc_uuid
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        for health_check in lb.health_check:
+            if health_check.uuid == self.hc_uuid:
+                lb.delvalue('health_check', health_check)
+                health_check.delete()
+                return
+        if not self.if_exists:
+            msg = "Health check '%s' on lb %s does not exist" % (
+                self.hc_uuid, self.lb)
+            raise RuntimeError(msg)
+
+
+class LbAddIpPortMappingСommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, endpoint_ip, port_name, source_ip):
+        super().__init__(api)
+        self.lb = lb
+        self.endpoint_ip = str(netaddr.IPAddress(endpoint_ip))
+        self.port_name = port_name
+        self.source_ip = str(netaddr.IPAddress(source_ip))
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        lb.setkey('ip_port_mappings', self.endpoint_ip,
+                  '%s:%s' % (self.port_name, self.source_ip))
+
+
+class LbDelIpPortMappingCommand(cmd.BaseCommand):
+    table = 'Load_Balancer'
+
+    def __init__(self, api, lb, endpoint_ip):
+        super().__init__(api)
+        self.lb = lb
+        self.endpoint_ip = str(netaddr.IPAddress(endpoint_ip))
+
+    def run_idl(self, txn):
+        lb = self.api.lookup(self.table, self.lb)
+        lb.delkey('ip_port_mappings', self.endpoint_ip)
+
+
+class HealthCheckAddCommand(cmd.AddCommand):
+    table_name = 'Load_Balancer_Health_Check'
+
+    def __init__(self, api, vip, **options):
+        super().__init__(api)
+        self.vip = utils.normalize_ip_port(vip)
+        self.options = options
+
+    def run_idl(self, txn):
+        hc = txn.insert(self.api.tables[self.table_name])
+        hc.vip = self.vip
+        hc.options = self.options
+        self.result = hc
+
+
+class HealthCheckSetOptionsCommand(cmd.BaseSetOptionsCommand):
+    table = 'Load_Balancer_Health_Check'
+
+
+class HealthCheckGetOptionsCommand(cmd.BaseGetOptionsCommand):
+    table = 'Load_Balancer_Health_Check'
+
+
 class LrLbAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, lb, may_exist=False):
         super().__init__(api)
@@ -1416,6 +1746,7 @@ class GatewayChassisAddCommand(cmd.AddCommand):
             # since 'name' is indexed
             gwc = txn.insert(self.api.tables[self.table_name])
             gwc.name = self.name
+        gwc.chassis_name = self.chassis_name
         gwc.priority = self.priority
         self.set_columns(gwc, **self.columns)
         self.result = gwc
