@@ -32,6 +32,8 @@ class OvsVenvFixture(fixtures.Fixture):
         os.path.join(os.path.sep, 'usr', 'local', 'share', 'openvswitch'),
         os.path.join(os.path.sep, 'usr', 'share', 'openvswitch'))
 
+    OVS_SCHEMA = 'vswitch.ovsschema'
+
     def __init__(self, venv, ovsdir=None, dummy=DUMMY_OVERRIDE_ALL,
                  remove=False):
         """Initialize fixture
@@ -76,10 +78,10 @@ class OvsVenvFixture(fixtures.Fixture):
 
     @property
     def ovs_schema(self):
-        path = os.path.join(self.ovsdir, 'vswitchd', 'vswitch.ovsschema')
+        path = os.path.join(self.ovsdir, 'vswitchd', self.OVS_SCHEMA)
         if os.path.isfile(path):
             return path
-        return os.path.join(self.ovsdir, 'vswitch.ovsschema')
+        return os.path.join(self.ovsdir, self.OVS_SCHEMA)
 
     @property
     def dummy_arg(self):
@@ -90,7 +92,7 @@ class OvsVenvFixture(fixtures.Fixture):
         return 'unix:' + os.path.join(self.venv, 'db.sock')
 
     def _setUp(self):
-        super(OvsVenvFixture, self)._setUp()
+        super()._setUp()
         self.addCleanup(self.deactivate)
         if not os.path.isdir(self.venv):
             os.mkdir(self.venv)
@@ -151,8 +153,10 @@ class OvsOvnVenvFixture(OvsVenvFixture):
         os.path.join(os.path.sep, 'usr', 'local', 'share', 'ovn'),
         os.path.join(os.path.sep, 'usr', 'share', 'ovn')) + (
             OvsVenvFixture.OVS_PATHS)
+
     NBSCHEMA = 'ovn-nb.ovsschema'
     SBSCHEMA = 'ovn-sb.ovsschema'
+    IC_NBSCHEMA = 'ovn-ic-nb.ovsschema'
 
     def __init__(self, venv, ovndir=None, add_chassis=False, **kwargs):
         self.add_chassis = add_chassis
@@ -160,7 +164,7 @@ class OvsOvnVenvFixture(OvsVenvFixture):
             # Use OVN source dir
             self.PATH_VAR_TEMPLATE += (
                 ":{0}/controller:{0}/northd:{0}/utilities".format(ovndir))
-        super(OvsOvnVenvFixture, self).__init__(venv, **kwargs)
+        super().__init__(venv, **kwargs)
         self.ovndir = self._share_path(self.OVN_PATHS, ovndir,
                                        [self.SBSCHEMA, self.NBSCHEMA])
         self.env.update({'OVN_RUNDIR': self.venv})
@@ -182,35 +186,35 @@ class OvsOvnVenvFixture(OvsVenvFixture):
         return 'unix:' + os.path.join(self.venv, 'ovnsb_db.sock')
 
     def setup_dbs(self):
-        super(OvsOvnVenvFixture, self).setup_dbs()
+        super().setup_dbs()
         self.create_db('ovnsb.db', self.ovnsb_schema)
         self.create_db('ovnnb.db', self.ovnnb_schema)
 
     def start_ovsdb_processes(self):
-        super(OvsOvnVenvFixture, self).start_ovsdb_processes()
-        self.call(['ovsdb-server', '--detach', '--no-chdir', '-vconsole:off',
-                   '--pidfile=%s' % os.path.join(self.venv, 'ovnnb_db.pid'),
-                   '--log-file=%s' % os.path.join(self.venv, 'ovnnb_db.log'),
-                   '--remote=db:OVN_Northbound,NB_Global,connections',
-                   '--private-key=db:OVN_Northbound,SSL,private_key',
-                   '--certificate=db:OVN_Northbound,SSL,certificate',
-                   '--ca-cert=db:OVN_Northbound,SSL,ca_cert',
-                   '--ssl-protocols=db:OVN_Northbound,SSL,ssl_protocols',
-                   '--ssl-ciphers=db:OVN_Northbound,SSL,ssl_ciphers',
-                   '--remote=p' + self.ovnnb_connection, 'ovnnb.db'])
-        self.call(['ovsdb-server', '--detach', '--no-chdir', '-vconsole:off',
-                   '--pidfile=%s' % os.path.join(self.venv, 'ovnsb_db.pid'),
-                   '--log-file=%s' % os.path.join(self.venv, 'ovnsb_db.log'),
-                   '--remote=db:OVN_Southbound,SB_Global,connections',
-                   '--private-key=db:OVN_Southbound,SSL,private_key',
-                   '--certificate=db:OVN_Southbound,SSL,certificate',
-                   '--ca-cert=db:OVN_Southbound,SSL,ca_cert',
-                   '--ssl-protocols=db:OVN_Southbound,SSL,ssl_protocols',
-                   '--ssl-ciphers=db:OVN_Southbound,SSL,ssl_ciphers',
-                   '--remote=p' + self.ovnsb_connection, 'ovnsb.db'])
+        super().start_ovsdb_processes()
+        data = [
+            (self.ovnnb_connection,
+             "OVN_Northbound", "ovnnb", "NB_Global"),
+            (self.ovnsb_connection,
+             "OVN_Southbound", "ovnsb", "SB_Global")]
+
+        for connection, schema, db_name, table in data:
+            self.call(['ovsdb-server',
+                       '--detach', '--no-chdir', '-vconsole:off',
+                       '--pidfile=%s' % os.path.join(self.venv,
+                                                     '%s_db.pid' % (db_name)),
+                       '--log-file=%s' % os.path.join(self.venv,
+                                                      '%s_db.log' % (db_name)),
+                       '--remote=db:%s,%s,connections' % (schema, table),
+                       '--private-key=db:%s,SSL,private_key' % (schema),
+                       '--certificate=db:%s,SSL,certificate' % (schema),
+                       '--ca-cert=db:%s,SSL,ca_cert' % (schema),
+                       '--ssl-protocols=db:%s,SSL,ssl_protocols' % (schema),
+                       '--ssl-ciphers=db:%s,SSL,ssl_ciphers' % (schema),
+                       '--remote=p' + connection, '%s.db' % (db_name)])
 
     def init_processes(self):
-        super(OvsOvnVenvFixture, self).init_processes()
+        super().init_processes()
         self.call(['ovn-nbctl', 'init'])
         self.call(['ovn-sbctl', 'init'])
         if self.add_chassis:
@@ -229,3 +233,81 @@ class OvsOvnVenvFixture(OvsVenvFixture):
                    '--ovnnb-db=' + self.ovnnb_connection])
         self.call(['ovn-controller', '--detach', '--no-chdir', '--pidfile',
                    '-vconsole:off', '--log-file'])
+
+
+class OvsOvnIcVenvFixture(OvsOvnVenvFixture):
+    def _setUp(self):
+        if self.has_icnb():
+            super()._setUp()
+
+    @property
+    def ovn_icnb_connection(self):
+        return 'unix:' + os.path.join(self.venv, 'ovn_ic_nb_db.sock')
+
+    @property
+    def ovn_icnb_schema(self):
+        return os.path.join(self.ovndir, self.IC_NBSCHEMA)
+
+    def has_icnb(self):
+        return os.path.isfile(self.ovn_icnb_schema)
+
+    def setup_dbs(self):
+        super().setup_dbs()
+        self.create_db('ovn_ic_nb.db', self.ovn_icnb_schema)
+
+    def start_ovsdb_processes(self):
+        super().start_ovsdb_processes()
+        connection, schema, db_name, table = (
+            self.ovn_icnb_connection,
+            "OVN_IC_Northbound", "ovn_ic_nb", "IC_NB_Global")
+        self.call(['ovsdb-server',
+                   '--detach', '--no-chdir', '-vconsole:off',
+                   '--pidfile=%s' % os.path.join(self.venv,
+                                                 '%s_db.pid' % (db_name)),
+                   '--log-file=%s' % os.path.join(self.venv,
+                                                  '%s_db.log' % (db_name)),
+                   '--remote=db:%s,%s,connections' % (schema, table),
+                   '--private-key=db:%s,SSL,private_key' % (schema),
+                   '--certificate=db:%s,SSL,certificate' % (schema),
+                   '--ca-cert=db:%s,SSL,ca_cert' % (schema),
+                   '--ssl-protocols=db:%s,SSL,ssl_protocols' % (schema),
+                   '--ssl-ciphers=db:%s,SSL,ssl_ciphers' % (schema),
+                   '--remote=p' + connection, '%s.db' % (db_name)])
+
+    def init_processes(self):
+        super().init_processes()
+        self.call(['ovn-ic-nbctl', 'init'])
+
+
+class OvsVtepVenvFixture(OvsOvnVenvFixture):
+    VTEP_SCHEMA = 'vtep.ovsschema'
+
+    def __init__(self, venv, vtepdir=None, **kwargs):
+        if vtepdir and os.path.isdir(vtepdir):
+            self.PATH_VAR_TEMPLATE += ":{0}".format(vtepdir)
+        self.vtepdir = self._share_path(self.OVS_PATHS, vtepdir)
+        super().__init__(venv, **kwargs)
+
+    def _setUp(self):
+        if self.has_vtep:
+            super()._setUp()
+
+    @property
+    def vtep_schema(self):
+        return os.path.join(self.vtepdir, self.VTEP_SCHEMA)
+
+    @property
+    def has_vtep(self):
+        return os.path.isfile(self.vtep_schema)
+
+    def setup_dbs(self):
+        db_filename = 'vtep.conf'
+        super().setup_dbs()
+        self.create_db(db_filename, self.vtep_schema)
+        self.ovsdb_server_dbs.append(db_filename)
+
+    def init_processes(self):
+        super().init_processes()
+        # there are no 'init' method in vtep-ctl,
+        # but record in 'Global' table is needed
+        self.call(['vtep-ctl', 'show'])
